<?php

/**
 * Plugin Name:       DriveWorks Shortcode - Form Embed
 * Plugin URI:        https://plugins.driveworkslive.com
 * Description:       Use shortcode to quickly embed DriveWorks Forms (Projects/DriveApps)
 * Author:            DriveWorks Ltd
 * Author URI:        https://driveworks.co.uk
 * Version:           1.0.1
 * License:           GPL v2 or later
 * License URI:       https://www.gnu.org/licenses/gpl-2.0.html
 */

defined( 'ABSPATH' ) || exit;
define( 'DRIVEWORKS_SHORTCODE_EMBED_VERSION', '1.0.1' );

add_action('wp_enqueue_scripts', 'dw_shortcode_embed_enqueue_assets');
add_shortcode('driveworks_embed', 'dw_shortcode_embed');

if ( !function_exists( 'dw_shortcode_embed' ) ) {
    function dw_shortcode_embed($atts) {
        extract($atts, EXTR_PREFIX_ALL, "dw_shortcode_embed");

        // Confirm required data is provided
        if ( empty($dw_shortcode_embed_server_url) ) {
            return dw_shortcode_embed_error("Missing required details: server_url");
        }
        if ( empty($dw_shortcode_embed_group_alias) ) {
            return dw_shortcode_embed_error("Missing required details: group_alias") . '</div></div>';
        }
        if ( empty($dw_shortcode_embed_project_name) && empty($dw_shortcode_embed_drive_app_alias) ) {
            return dw_shortcode_embed_error("Missing required details: project_name or drive_app_alias");
        }

        // Sanitize server url
        $dw_shortcode_embed_server_url = htmlspecialchars(rtrim($dw_shortcode_embed_server_url, "/ "));

        // Check client url is valid
        $dw_shortcode_embed_client_url = $dw_shortcode_embed_server_url . "/DriveWorksLiveIntegrationClient.min.js";
        if (!dw_shortcode_embed_file_exists($dw_shortcode_embed_client_url)) {
            return dw_shortcode_embed_error("Client library not found at configured server_url");
        }

        // Load dynamic client script - using configured server url
        wp_enqueue_script('driveworks-live-integration-client', $dw_shortcode_embed_client_url);

        // Generate markup
        $dw_shortcode_embed_form_message_loading = "Loading...";
        $dw_shortcode_embed_form_message_complete = "Thank you for your submission.";
        $dw_shortcode_embed_form_message_cancel = "This submission has been cancelled.";

        $dw_shortcode_embed_markup = '<div data-driveworks-shortcode-embed class="dw-shortcode-embed dw-is-loading"' . (is_user_logged_in() ? ' data-debug="true"' : '');
        $dw_shortcode_embed_markup .= ' data-server-url="' . $dw_shortcode_embed_server_url . '"';
        $dw_shortcode_embed_markup .= ' data-group-alias="' . htmlspecialchars($dw_shortcode_embed_group_alias) . '"';
        $dw_shortcode_embed_project_name ? $dw_shortcode_embed_markup .= ' data-project-name="' . htmlspecialchars($dw_shortcode_embed_project_name) . '"' : "";
        $dw_shortcode_embed_drive_app_alias ? $dw_shortcode_embed_markup .= ' data-drive-app-alias="' . htmlspecialchars($dw_shortcode_embed_drive_app_alias) . '"' : "";
        $dw_shortcode_embed_ping_interval ? $dw_shortcode_embed_markup .= ' data-ping-interval="' . htmlspecialchars($dw_shortcode_embed_ping_interval) . '"' : "";
        $dw_shortcode_embed_markup .= '>
            <div data-form-output class="dw-form-output" style="height: ' . ($dw_shortcode_embed_height ? htmlspecialchars($dw_shortcode_embed_height) : 'auto') . ';">
                <div class="dw-embed-loading dw-embed-message">' . $dw_shortcode_embed_form_message_loading . '</div>
            </div>
            <div class="dw-message-form-complete" data-message="' . $dw_shortcode_embed_form_message_complete . '"></div>
            <div class="dw-message-form-cancel" data-message="' . $dw_shortcode_embed_form_message_cancel . '"></div>
        </div>
        ';

        return $dw_shortcode_embed_markup;
    }
}

/** Enqueue client script + embed styles */
if ( !function_exists( 'dw_shortcode_embed_enqueue_assets' ) ) {
    function dw_shortcode_embed_enqueue_assets() {
        global $post;
        if (is_a($post, 'WP_Post') && has_shortcode($post->post_content, 'driveworks_embed')) {

            // Load embed styles
            wp_register_style('dw-shortcode-embed-style', plugins_url('style.css', __FILE__));
            wp_enqueue_style('dw-shortcode-embed-style');

            // Load embed scripts
            wp_register_script('dw-shortcode-embed-script', plugins_url('script.js', __FILE__));
            wp_enqueue_script('dw-shortcode-embed-script');
        }
    }
}

/** Check remote files exists */
if ( !function_exists( 'dw_shortcode_embed_file_exists' ) ) {
    function dw_shortcode_embed_file_exists($url) {
        try {
            $headers = get_headers($url, 1);
            if ( !is_array($headers) ) { return false; }

            $statuses = array_filter(
                $headers,
                function ($key) {
                  return is_numeric($key);
                },
                ARRAY_FILTER_USE_KEY
            );
            $finalStatus = end($statuses);
            return stripos($finalStatus, "200 OK") ? true : false;
        } catch (exception $e) {
            return false;
        }
    }
}

/** Render alternate error message based on user login status */
if ( !function_exists( 'dw_shortcode_embed_error' ) ) {
    function dw_shortcode_embed_error($message) {
        $error_text = "There has been a problem.";
        if (is_user_logged_in()) {
            $error_text = "<strong>[DriveWorks Shortcode Embed]</strong> " . $message;
        }

        return '<div class="dw-shortcode-embed"><div class="dw-embed-error">' . $error_text . '</div></div>';
    }
}
